"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.router = exports.Route = exports.Response = exports.Request = exports.Method = void 0;
var http_1 = require("http");
exports.Method = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    DELETE: 'DELETE',
};
var Request = /** @class */ (function (_super) {
    __extends(Request, _super);
    function Request() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return Request;
}(http_1.IncomingMessage));
exports.Request = Request;
var Response = /** @class */ (function (_super) {
    __extends(Response, _super);
    function Response() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return Response;
}(http_1.ServerResponse));
exports.Response = Response;
var Router = /** @class */ (function () {
    function Router() {
        this.routes = {};
        this.routes[exports.Method.GET] = [];
    }
    Router.prototype.registerRoutes = function (callback) {
        callback(this);
    };
    Router.prototype.dispatch = function (req, res) {
        if (req.method === undefined) {
            res.statusCode = 404;
            res.end('404 Not Found');
            return;
        }
        var potentialRoutes = this.routes[req.method].filter(function (route) { return route.isMatch(req); });
        if (potentialRoutes.length > 0) {
            potentialRoutes[0].action(req, res);
            return;
        }
        res.statusCode = 404;
        res.end('404 Not Found');
    };
    Router.prototype.get = function (path, action) {
        this.routes[exports.Method.GET].push(new Route(exports.Method.GET, path, action));
    };
    return Router;
}());
var Route = /** @class */ (function () {
    function Route(method, path, action) {
        this.method = method;
        this.path = path;
        this.action = action;
        this.regex = this.buildRegex();
    }
    Route.prototype.isMatch = function (req) {
        return req.url !== undefined ? this.regex.test(req.url) : false;
    };
    Route.prototype.buildRegex = function () {
        var segments = this.path.split('/')
            .map(function (segment) {
            if (segment.charAt(0) === '{' && segment.charAt(segment.length - 1) === '}') {
                return "(?<" + segment.slice(1, -1) + ">[A-Za-z0-9]+)";
            }
            return segment;
        })
            .join('/');
        return new RegExp("^" + segments + "$");
    };
    return Route;
}());
exports.Route = Route;
exports.router = new Router();
